@@ -1,7 +1,7 @@
 
 /*
  *  This file is part of Complexity.
- *  Complexity Copyright (c) 2011, 2014 by Bruce Korb - all rights reserved
+ *  Complexity Copyright (c) 2011-2016 by Bruce Korb - all rights reserved
  *
  *  Complexity is free software: you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
@@ -22,10 +22,22 @@
 
 static jmp_buf bail_on_proc;
 
-static char const err_fmt[] = "error: %s %s\n";
+static char const err_fmt[]    = "error: %s %s\n";
+static char const line_score[] = "line %5d score %5u\n";
 
 typedef score_t (handler_func_t)(state_t *);
 
+/**
+ * subexpression types that have been seen.  Tracking is only done within
+ * one parenthesized expression.  If two or more types are mixed, the
+ * expression gets penalized.  If there is ever an assignment within
+ * an expression, the expression is penalized.
+ */
+typedef struct {
+    int saw_and, saw_or, saw_assign, saw_relop;
+    score_t res;
+} subexpr_seen_t;
+
 /*
  * declare the dispatch functions and define the table.
  */
@@ -47,6 +59,7 @@ typedef score_t (handler_func_t)(state_t *);
 #define handle_TKN_LIT_CLSPAREN handle_invalid
 #define handle_TKN_LIT_COMMA    handle_noop
 #define handle_TKN_LIT_COLON    handle_invalid
+#define handle_TKN_LIT_QUESTION handle_expression
 #define handle_TKN_LIT_CLSBRACK handle_invalid
 #define handle_TKN_LIT_OPNBRACK handle_array_init
 #define handle_TKN_LIT_CBRACE   handle_invalid
@@ -64,6 +77,10 @@ static handler_func_t * score_fun[TOKEN_MAX] = {
 #undef  _Ttbl_
 };
 
+#define APPLY_NEST_PENALTY(_s)   ((_s) * penalty)
+
+static int statement_depth = 0;
+
 static score_t
 handle_subexpr(state_t * sc, bool is_for_clause);
 
@@ -87,20 +104,19 @@ bad_token(state_t * sc, char const * where, token_t ev)
         p = invbuf;
     }
 
-    fprintf(stderr, msgfmt, sc->ln_ct, sc->pname, sc->fstate->fname,
-            sc->proc_line + sc->ln_ct, where, p, ev);
+    fprintf(stderr, msgfmt, sc->st_line_ct, sc->pname, sc->st_fstate->fs_fname,
+            sc->proc_line + sc->st_line_ct, where, p, ev);
     return MAX_SCORE;
 }
 
 static token_t
 next_score_token(state_t * sc)
 {
-    token_t tk = next_token(sc->fstate);
-    if ((tk == TKN_EOF) || (sc->fstate->scan > sc->end))
-        longjmp(bail_on_proc, 1);
+    fstate_t * fs = sc->st_fstate;
+    token_t    tk = next_token(fs);
 
-    sc->ln_ct   = sc->fstate->cur_line - sc->ln_st;
-    sc->ncln_ct = sc->fstate->nc_line  - sc->ncln_st;
+    if ((tk == TKN_EOF) || (fs->fs_scan > sc->st_end))
+        longjmp(bail_on_proc, 1);
 
     if (tk != TKN_KW_GOTO)
         return tk;
@@ -111,21 +127,37 @@ next_score_token(state_t * sc)
 static void
 unget_score_token(state_t * sc)
 {
-    unget_token(sc->fstate);
+    unget_token(sc->st_fstate);
 }
 
+/**
+ * The open brace for the statement block has already been processed.
+ */
 static score_t
 handle_stmt_block(state_t * sc)
 {
+    fstate_t * fs = sc->st_fstate;
     score_t res = 0;
 
-    for (;;) {
-        token_t ev = next_score_token(sc);
+    /*
+     * Bookwork for the first call for a procedure
+     */
+    token_t ev = next_score_token(sc);
+    if (sc->st_nc_line_ct < 0) {
+        sc->st_line_ct    = fs->cur_line;
+        sc->st_nc_line_ct = fs->nc_line;
+    }
+
+    if (++statement_depth > sc->st_depth_warned)
+        sc->st_depth_warned = statement_depth;
+
+    for (;; ev = next_score_token(sc)) {
         switch (ev) {
         case TKN_LIT_CBRACE:
             if (HAVE_OPT(TRACE))
-                fprintf(trace_fp, "line %5d score %5u\n",
-                        sc->fstate->cur_line, (unsigned int)res);
+                fprintf(trace_fp, line_score,
+                        fs->cur_line, (unsigned int)res);
+            statement_depth--;
             return (res > MAX_SCORE) ? MAX_SCORE : res;
 
         default:
@@ -137,7 +169,7 @@ handle_stmt_block(state_t * sc)
 static score_t
 handle_invalid(state_t * sc)
 {
-    sc->fstate->scan = sc->end;
+    sc->st_fstate->fs_scan = sc->st_end;
     fprintf(stderr, "invalid transition\n");
     return MAX_SCORE;
 }
@@ -148,12 +180,7 @@ handle_noop(state_t * sc)
     return 0;
 }
 
-typedef struct {
-    int saw_and, saw_or, saw_assign, saw_relop;
-    score_t res;
-} subexpr_seen_t;
-
-static void
+static char const *
 fiddle_subexpr_score(subexpr_seen_t * ses)
 {
     int which =
@@ -168,11 +195,11 @@ fiddle_subexpr_score(subexpr_seen_t * ses)
     case 0x01:
     case 0x02:
     case 0x08:
-        return;
+        return NULL;
 
     case 0x04:
         ses->res += penalty * (score_t)ses->saw_assign;
-        return;
+        return "assignment within expression";
 
     case 0x07:
         ses->res += penalty * (score_t)ses->saw_assign;
@@ -181,49 +208,59 @@ fiddle_subexpr_score(subexpr_seen_t * ses)
     case 0x03:
         tmp = (ses->saw_and + 1) * ses->saw_or;
         ses->res += penalty * (score_t)tmp;
-        return;
+        return "AND and OR expressions";
 
     case 0x05:
     case 0x06:
         ses->res += penalty * (score_t)ses->saw_assign;
         ses->res += (score_t)(ses->saw_and + ses->saw_or);
-        return;
+        return "assignments and boolean operators";
 
     case 0x09:
     case 0x0A:
         ses->res += subexp_penalty * (score_t)ses->saw_relop;
-        return;
+        return "comparison and boolean operators";
 
     case 0x0B:
         tmp = (ses->saw_and + 1) * ses->saw_or;
         ses->res += subexp_penalty * (score_t)(ses->saw_relop * tmp);
-        return;
+        return "AND, OR and comparison operators";
 
     case 0x0C:
         ses->res += penalty * (score_t)ses->saw_assign;
         ses->res += subexp_penalty * (score_t)ses->saw_relop;
-        return;
+        return "assignments and comparison operators";
 
     case 0x0D:
     case 0x0E:
         ses->res += penalty * (score_t)(
             ses->saw_assign + ses->saw_relop + ses->saw_and + ses->saw_or);
-        return;
+        return "many kinds of operators";
 
     case 0x0F:
         tmp = (ses->saw_and + 1) * ses->saw_or;
         ses->res += penalty * (score_t)(
             ses->saw_assign + ses->saw_relop + tmp);
+        return "*ALL* kinds of operators";
+
+    default:
+        CX_ASSERT(which == 0);
     }
 }
 
+static score_t
+handle_TKN_LIT_SEMI(state_t * sc)
+{
+    return 1;
+}
+
 static score_t
 handle_subexpr(state_t * sc, bool is_for_clause)
 {
-    int saw_name = 0;
-    int stline   = sc->ncln_ct;
+    bool saw_name = false;
+    int  tkn_ct   =  0;
     subexpr_seen_t ses = { 0, 0, 0, 0, 1.0 };
-    int tkn_ct   =  0;
+    int const start_nc_ln_ct = sc->st_fstate->nc_line;
 
     for (;;) {
         token_t ev = next_score_token(sc);
@@ -234,14 +271,19 @@ handle_subexpr(state_t * sc, bool is_for_clause)
             if (tkn_ct <= 2) // name + close paren
                 return 0;
 
-            if (! is_for_clause)
-                fiddle_subexpr_score(&ses);
-            ses.res += (score_t)(sc->ncln_ct - stline);
+            if (! is_for_clause) {
+                char const * msg = fiddle_subexpr_score(&ses);
+                if ((msg != NULL) && HAVE_OPT(TRACE))
+                    fprintf(trace_fp, "line %5d expression score adjusted due "
+                            "to mix of %s\n", sc->st_fstate->cur_line, msg);
+            }
+
+            ses.res += (score_t)(sc->st_fstate->nc_line - start_nc_ln_ct);
             if (ses.res > 1)
                 ses.res -= 1;
             if (HAVE_OPT(TRACE))
-                fprintf(trace_fp, "line %5d score %u\n",
-                        sc->fstate->cur_line, (unsigned int)ses.res);
+                fprintf(trace_fp, line_score,
+                        sc->st_fstate->cur_line, (unsigned int)ses.res);
             return ses.res;
 
         case TKN_LIT_OPNPAREN:
@@ -261,7 +303,10 @@ handle_subexpr(state_t * sc, bool is_for_clause)
             break;
 
         case TKN_ASSIGN:
-            ses.saw_assign++;
+            /*
+             * Only worry over assignment operators in nested expressions
+             */
+            ses.saw_assign += (statement_depth > 1) ? 1 : 0;
             break;
 
         case TKN_LOGIC_AND:
@@ -285,10 +330,11 @@ handle_subexpr(state_t * sc, bool is_for_clause)
         case TKN_NUMBER:
         case TKN_ARITH_OP:
         case TKN_LIT_COLON:
+        case TKN_LIT_QUESTION:
             break;
 
         case TKN_LIT_OBRACE:
-            ses.res += NESTING_PENALTY(handle_stmt_block(sc));
+            ses.res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
             break;
 
         case TKN_LIT_OPNBRACK:
@@ -312,14 +358,19 @@ static score_t
 handle_parms(state_t * sc)
 {
     score_t res = 0;
-    int stline = sc->ncln_ct - 1;
-    int paren_is_parms = 1;
+    int const start_nc_ln_ct = sc->st_fstate->nc_line;
 
     for (;;) {
         token_t ev = next_score_token(sc);
         switch (ev) {
         case TKN_LIT_CLSPAREN:
-            return (score_t)(sc->ncln_ct - stline) + res;
+        {
+            /*
+             * Set to zero if we are on the same non-comment line count
+             */
+            int l_ct_delta = sc->st_fstate->nc_line - start_nc_ln_ct;
+            return (score_t)l_ct_delta + res;
+        }
 
         case TKN_LIT_OPNPAREN:
         {
@@ -330,7 +381,7 @@ handle_parms(state_t * sc)
         }
 
         case TKN_LIT_OBRACE:
-            res += NESTING_PENALTY(handle_stmt_block(sc));
+            res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
             break;
 
         case TKN_LIT_OPNBRACK:
@@ -348,16 +399,22 @@ handle_parms(state_t * sc)
     }
 }
 
+/**
+ * expressions always count at least 1.  Additional points
+ * are added for multiple lines and complexity and lack
+ * of parentheses when there are operators of different
+ * precedence.
+ */
 static score_t
 handle_expression(state_t * sc)
 {
     score_t res = 1;
-    int stline  = sc->ncln_ct;
+    int const start_nc_ln_ct = sc->st_fstate->nc_line;
     bool paren_is_parms    = true;
     bool cbrace_needs_semi = false;
 
     for (;;) {
-        token_t ltk = sc->fstate->last_tkn;
+        token_t ltk = sc->st_fstate->last_tkn;
         token_t ev = next_score_token(sc);
         bool next_paren_is_parms    = false;
         bool next_cbrace_needs_semi = false;
@@ -365,8 +422,8 @@ handle_expression(state_t * sc)
         switch (ev) {
         case TKN_LIT_CBRACE:
             if (HAVE_OPT(TRACE))
-                fprintf(trace_fp, "line %5d score %5u\n",
-                        sc->fstate->cur_line, (unsigned int)res);
+                fprintf(trace_fp, line_score,
+                        sc->st_fstate->cur_line, (unsigned int)res);
             /* FALLTHROUGH */
         case TKN_LIT_CLSBRACK:
         case TKN_LIT_CLSPAREN:
@@ -395,7 +452,7 @@ handle_expression(state_t * sc)
             break;
 
         case TKN_LIT_OBRACE:
-            res += NESTING_PENALTY(handle_stmt_block(sc));
+            res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
             if (! cbrace_needs_semi) {
                 /*
                  * Someone has mutilated C syntax and made a macro into
@@ -403,7 +460,7 @@ handle_expression(state_t * sc)
                  * To fix this, we lie to our caller and claim we saw a
                  * semi-colon.
                  */
-                sc->fstate->last_tkn = TKN_LIT_SEMI;
+                sc->st_fstate->last_tkn = TKN_LIT_SEMI;
                 goto done;
             }
             break;
@@ -418,12 +475,19 @@ handle_expression(state_t * sc)
 
         case TKN_LIT_COLON:
             /*
-             * A name followed by a colon is a statement label.
+             * A name followed by a colon is a statement label, unless
+             * we are in the middle of a ternary operation.
              */
-            if (ltk ==TKN_NAME)
+            if (sc->st_colon_need > 0) {
+                sc->st_colon_need--;
+            } else if (ltk ==TKN_NAME)
                 goto done;
             break;
 
+        case TKN_LIT_QUESTION:
+            sc->st_colon_need++;
+            break;
+
         case TKN_NUMBER:
         case TKN_ARITH_OP:
         default:
@@ -436,7 +500,11 @@ handle_expression(state_t * sc)
 
 done:
     {
-        score_t min = 1 + (sc->ncln_ct - stline);
+        /*
+         * Set the minimum to the number of lines used by the expr
+         * and limit the maximum to MAX_SCORE
+         */
+        score_t min = 1 + (sc->st_fstate->nc_line - start_nc_ln_ct);
         if (res < min)
             res = min;
         else if (res > MAX_SCORE)
@@ -514,7 +582,7 @@ handle_TKN_KW_DO(state_t * sc)
 
     switch (ev) {
     case TKN_LIT_OBRACE:
-        res += NESTING_PENALTY(handle_stmt_block(sc));
+        res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
         break;
 
     case TKN_LIT_OPNPAREN:
@@ -526,7 +594,7 @@ handle_TKN_KW_DO(state_t * sc)
     case TKN_KW_FOR:
     case TKN_KW_SWITCH:
     case TKN_KW_WHILE:
-        res += NESTING_PENALTY(score_fun[ev](sc));
+        res += APPLY_NEST_PENALTY(score_fun[ev](sc));
         break;
 
     default:
@@ -542,7 +610,7 @@ handle_TKN_KW_DO(state_t * sc)
         return bad_token(sc, "while loop expression", ev);
 
     {
-        int psc = NESTING_PENALTY(handle_subexpr(sc, false));
+        int psc = APPLY_NEST_PENALTY(handle_subexpr(sc, false));
         if (psc < 1)
             psc = 1;
         res += psc;
@@ -576,7 +644,7 @@ do_if_clause:
 
     switch (ev) {
     case TKN_LIT_OBRACE:
-        res += NESTING_PENALTY(handle_stmt_block(sc));
+        res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
         break;
 
     case TKN_LIT_SEMI:
@@ -605,18 +673,17 @@ do_if_clause:
     case TKN_KW_FOR:
     case TKN_KW_SWITCH:
     case TKN_KW_WHILE:
-        res += NESTING_PENALTY(score_fun[ev](sc));
+        res += APPLY_NEST_PENALTY(score_fun[ev](sc));
         break;
 
     default:
-    bad_token:
         return bad_token(sc, "bad if block", ev);
     }
 
     if (then_clause_done)
         return res;
 
-    ev = sc->fstate->last_tkn;
+    ev = sc->st_fstate->last_tkn;
     {
         token_t ntkn = next_score_token(sc);
         if (ntkn != TKN_KW_ELSE) {
@@ -624,7 +691,7 @@ do_if_clause:
              * Put the token back.  The caller will need it.
              */
             unget_score_token(sc);
-            sc->fstate->last_tkn = ev;
+            sc->st_fstate->last_tkn = ev;
             return res;
         }
     }
@@ -641,7 +708,7 @@ static score_t
 handle_TKN_KW_FOR(state_t * sc)
 {
     score_t res;
-    bool    real_for = (sc->fstate->last_tkn == TKN_KW_FOR);
+    bool    real_for = (sc->st_fstate->last_tkn == TKN_KW_FOR);
     token_t ev = next_score_token(sc);
 
     if (ev != TKN_LIT_OPNPAREN)
@@ -651,13 +718,13 @@ handle_TKN_KW_FOR(state_t * sc)
     if (res < 1)
         res = 1;
     if (! real_for)
-        res = NESTING_PENALTY(res);
+        res *= penalty;
 
     for (;;) {
         ev = next_score_token(sc);
         switch (ev) {
         case TKN_LIT_OBRACE:
-            res += NESTING_PENALTY(handle_stmt_block(sc));
+            res += APPLY_NEST_PENALTY(handle_stmt_block(sc));
             return res;
 
         case TKN_LIT_SEMI:
@@ -668,7 +735,7 @@ handle_TKN_KW_FOR(state_t * sc)
         case TKN_KW_FOR:
         case TKN_KW_SWITCH:
         case TKN_KW_WHILE:
-            res += NESTING_PENALTY(score_fun[ev](sc));
+            res += APPLY_NEST_PENALTY(score_fun[ev](sc));
             return res;
 
         case TKN_LIT_OPNPAREN:
@@ -682,21 +749,19 @@ handle_TKN_KW_FOR(state_t * sc)
         case TKN_NAME:
         case TKN_NUMBER:
             res += handle_expression(sc);
-            if (sc->fstate->last_tkn == TKN_LIT_SEMI)
+            if (sc->st_fstate->last_tkn == TKN_LIT_SEMI)
                 return res;
-            if (sc->fstate->last_tkn != TKN_LIT_COMMA)
+            if (sc->st_fstate->last_tkn != TKN_LIT_COMMA)
                 return bad_token(sc, "loop body ended badly",
-                                 sc->fstate->last_tkn);
+                                 sc->st_fstate->last_tkn);
+            /* FALLTHROUGH */
+
+        default:
+            continue;
         }
     }
 }
 
-static score_t
-handle_TKN_LIT_SEMI(state_t * sc)
-{
-    return 1;
-}
-
 static score_t
 handle_bracket_expr(state_t * sc)
 {
@@ -704,7 +769,7 @@ handle_bracket_expr(state_t * sc)
 
     for (;;) {
         res += handle_expression(sc);
-        if (sc->fstate->last_tkn != TKN_LIT_COMMA)
+        if (sc->st_fstate->last_tkn != TKN_LIT_COMMA)
             break;
         res -= 1;
     }
@@ -729,6 +794,22 @@ handle_array_init(state_t * sc)
     return res + handle_expression(sc);
 }
 
+/**
+ * return 'true' if the closing brace is on its own line.
+ * This affects both the non-comment line count and
+ * the total line count.
+ */
+static inline bool
+check_own_line_close(state_t * sc)
+{
+    char const * p = sc->st_fstate->fs_scan - 1;
+    CX_ASSERT(*p == '}');
+    return IS_END_OF_LINE_CHAR(p[-1]);
+}
+
+/**
+ * Score the procedure we just found.
+ */
 void
 score_proc(state_t * score)
 {
@@ -745,29 +826,59 @@ score_proc(state_t * score)
 
     if (setjmp(bail_on_proc) != 0) {
         fprintf(stderr, "end of %s() in %s reached with open control blocks\n",
-                score->pname, score->fstate->fname);
+                score->pname, score->st_fstate->fs_fname);
 
         score->score = MAX_SCORE;
         return;
     }
 
+    statement_depth = 0;
+    score->st_line_ct    = \
+        score->st_nc_line_ct = -1;
+
     score->score = handle_stmt_block(score);
     if (score->goto_ct > 0)
         score->score += score->goto_ct * scaling;
 
-    if (score->fstate->scan + 2 <= score->end) {
+    if (score->st_depth_warned >= 5) {
+        fprintf(stderr, "NOTE: proc %s in file %s line %u\n"
+                "\tnesting depth reached level %u\n",
+                score->pname, score->st_fstate->fs_fname,
+                score->proc_line, score->st_depth_warned);
+        if (score->st_depth_warned >= 7)
+            fputs("==>\t*seriously consider rewriting the procedure*.\n", stderr);
+    }
+
+    if (score->st_fstate->fs_scan + 2 <= score->st_end) {
         fprintf(stderr, "procedure %s in %s ended before final close bracket\n",
-                score->pname, score->fstate->fname);
+                score->pname, score->st_fstate->fs_fname);
 
         score->score += penalty;
     }
 
-    if (score->score < MAX_SCORE)
+    /*
+     * No negative scores and no scores beyond the completely ridiculous
+     * value of MAX_SCORE
+     */
+    if (score->score < 0)
+        score->score = 0.0;
+
+    else if (score->score < MAX_SCORE)
         score->score = (score_t)(unsigned int)((score->score * scaling) + 0.9);
-    if (--score->ln_ct <= 0)
-        score->ln_ct = 1;
-    if (--score->ncln_ct <= 0)
-        score->ncln_ct = 1;
+
+    if (score->score > MAX_SCORE)
+        score->score = MAX_SCORE;
+
+    /*
+     * Set the line counts for our score
+     */
+    bool close_on_own_line = check_own_line_close(score);
+    int ct = 1 + (score->st_fstate->cur_line - score->st_line_ct) -
+        (close_on_own_line ? 1 : 0);
+    score->st_line_ct    = ct;
+    ct = 1 + (score->st_fstate->nc_line  - score->st_nc_line_ct) -
+        (close_on_own_line ? 1 : 0);
+    score->st_nc_line_ct = ct;
 }
 /*
  * Local Variables:
